// add.js
//////////////////////////////////////////////////////////////////////////////////////

Bigint.add = function( number1, number2 ) {
	var bigNumber1 = Bigint.serio( number1 );
	var bigNumber2 = Bigint.serio( number2 );
	var cmp;
	var bigResult;

	if( bigNumber1.isNonNegative ) {
		if( bigNumber2.isNonNegative ) {
			bigResult = Bigint._add( bigNumber1, bigNumber2 );
		} else {
			cmp = Bigint._tablecmp( bigNumber1.table, bigNumber2.table );
			if( cmp > 0 ) {
				bigResult = Bigint._sub( bigNumber1, bigNumber2 );
			} else if( cmp === 0 ) {
				return Bigint( 0 );
			} else {
				bigResult = Bigint._sub( bigNumber2, bigNumber1 );
				bigResult.isNonNegative = false;
			}
		}
	} else {
		if( bigNumber2.isNonNegative ) {
			cmp = Bigint._tablecmp( bigNumber1.table, bigNumber2.table );
			if( cmp > 0 ) {
				bigResult = Bigint._sub( bigNumber1, bigNumber2 );
				bigResult.isNonNegative = false;
			} else if( cmp === 0 ) {
				return Bigint( 0 );
			} else {
				bigResult = Bigint._sub( bigNumber2, bigNumber1 );
			}
		} else {
			bigResult = Bigint._add( bigNumber1, bigNumber2 );
			bigResult.isNonNegative = false;
		}
	}
	return bigResult;
}

Bigint._add = function( bigNumber1, bigNumber2 ) {
	if( bigNumber1.table.length > bigNumber2.table.length ) {
		var table1 = bigNumber1.table;
		var table2 = bigNumber2.table;
	} else {
		var table1 = bigNumber2.table;
		var table2 = bigNumber1.table;
	}

	var nLength1 = table1.length, nLength2 = table2.length;
	var tableResult = new Array( nLength1 );

	var i = 0, flag = 0;
	for( ; i < nLength2; i++ ) {
		tableResult[ i ] = table1[ i ] + table2[ i ] + flag;
		if( tableResult[ i ] >= __BIGINT_UNIT ) {
			flag = 1;
			tableResult[ i ] -= __BIGINT_UNIT;
		} else {
			flag = 0;
		}
	}

	for( ; i < nLength1 && flag; i++ ) {
		tableResult[ i ] = table1[ i ] + flag;
		if( tableResult[ i ] === __BIGINT_UNIT ) {
			tableResult[ i ] = 0;
			flag = 1;
		} else {
			flag = 0;
		}
	}

	for( ; i < nLength1; i++ ) {
		tableResult[ i ] = table1[ i ];
	}

	if( flag ) tableResult[ i ] = 1;

	var bigResult = new Bigint();
	bigResult.table = tableResult;
	return bigResult;
}


Bigint._add_typeN = function( bigNumber, smallNumber ) { // bigNumber must be > 0 && smallNumber should be <= __BIGINT_UNIT
	var bigResult = bigNumber.copy();
	bigResult.table[ 0 ] += smallNumber;
	if( bigResult.table[ 0 ] < __BIGINT_UNIT ) return bigResult;
	bigResult.table[ 0 ] -= __BIGINT_UNIT;
	var i = 1, len = bigResult.table.length;
	for( ; i < len; i++ ) {
		bigResult.table[ i ]++;
		if( bigResult.table[ i ] < __BIGINT_UNIT ) return bigResult;
		bigResult.table[ i ] = 0;
	}
	bigResult.table[ i ] = 1;
	return bigResult;
}

Bigint.prototype.increment = function( input ) {
	if( input === undefined ) {
		if( this.isNonNegative ) this._increment( 1 );
		else {
			this._decrement( 1 );
			this.isNonNegative = this.isZero();
		}
	} else if( input.isBigint || input > __BIGINT_UNIT || input < 0 || !this.isNonNegative ) {
		var bigResult = Bigint.add( this , input );
		this.table = bigResult.table;
		this.isNonNegative = bigResult.isNonNegative;
	} else {
		var n = Math.floor( input );
		if( isNaN( n ) ) {
			Bigint._error( "increment" , "Bad input: " + input );
		} else {
			this._increment( n );
		}
	}
}

Bigint.prototype._increment = function( smallNumber ) { // smallNumber should be <= __BIGINT_UNIT
	if( smallNumber === undefined ) smallNumber = 1;
	table = this.table;
	table[ 0 ] += smallNumber;
	if( table[ 0 ] < __BIGINT_UNIT ) return;
	else table[ 0 ] -= __BIGINT_UNIT;
	var i = 1;
	var len = table.length;
	for( ; i < len; i++ ) {
		table[ i ]++;
		if( table[ i ] < __BIGINT_UNIT ) return;
		else table[ i ] = 0;
	}
	table[ i ] = 1;
}

Bigint._stradd = function( strNumber1, strNumber2 ) {
	var biggerTable = new Array();
	var n1, n2;
	var flag = 0;
	var i = 0;
	var pos1 = strNumber1.length;
	var pos2 = strNumber2.length;
	for( ; pos1 > 0 || pos2 > 0; pos1 -= __BIGINT_BIGGER_POWER, pos2 -= __BIGINT_BIGGER_POWER, i++ ) {
		n1 = Number( strNumber1.substring( pos1 - __BIGINT_BIGGER_POWER, pos1 ) );
		n2 = Number( strNumber2.substring( pos2 - __BIGINT_BIGGER_POWER, pos2 ) );
		var nResult = n1 + n2 + flag;
		if( nResult >= __BIGINT_BIGGER_UNIT ) {
			biggerTable[ i ] = nResult - __BIGINT_BIGGER_UNIT;
			flag = 1;
		} else {
			biggerTable[ i ] = nResult;
			flag = 0;
		}
	}
	if( flag ) biggerTable[ i ] = 1;
	var lastIndex = biggerTable.length - 1;
	var strResult = String( biggerTable[ lastIndex ] );
	for( var i = lastIndex - 1; i > -1; i-- ) {
		strResult += Bigint._paddingBigger( biggerTable[ i ] );
	}
	return strResult;
}

Bigint._add_typeA = function( strNumber1, strNumber2, nAccuracy ) {
	var len1 = strNumber1.length;
	var len2 = strNumber2.length;
	var nZeros;
	if( len1 > len2 ) {
		var s = nAccuracy - ( len1 - len2 );
		if( s <= 0 ) return strNumber1;
		var strA = strNumber1.substr( 0, nAccuracy );
		var strB = strNumber2.substr( 0, s );
		nZeros = len1 - nAccuracy;
	} else {
		var s = nAccuracy - ( len2 - len1 );
		if( s <= 0 ) return strNumber2;

		var strA = strNumber2.substr( 0, nAccuracy );
		var strB = strNumber1.substr( 0, s );
		nZeros = len2 - nAccuracy;
	}
	if( nZeros > 0 ) return Bigint._stradd( strA, strB ) + Bigint._getStringZeros( nZeros );
	else return Bigint._stradd( strA, strB );
}