// div.js


//////////////////////////////////////////////////////////////////////////////////////

Bigint.prototype.half = function() {
	var table = this.table;
	var nLastIndex = table.length - 1;
	var flag = false;
	for( var i = nLastIndex; i > -1; i-- ) {
		if( table[ i ] & 1 ) {
			table[ i ]--;
			table[ i ] >>= 1;
			if( flag ) table[ i ] += __BIGINT_HALF_UNIT;
			flag = true;
		} else {
			table[ i ] >>= 1;
			if( flag ) table[ i ] += __BIGINT_HALF_UNIT;
			flag = false;
		}
	}
	if( table[ nLastIndex ] === 0 && nLastIndex > 0 ) table.length--;
	return flag; // This function should return Boolean( is this odd? )
}

Bigint._tableHalf = function( table ) {
	var nLastIndex = table.length - 1;
	var flag = false;
	for( var i = nLastIndex; i > -1; i-- ) {
		if( table[ i ] & 1 ) {
			table[ i ]--;
			table[ i ] >>= 1;
			if( flag ) table[ i ] += __BIGINT_HALF_UNIT;
			flag = true;
		} else {
			table[ i ] >>= 1;
			if( flag ) table[ i ] += __BIGINT_HALF_UNIT;
			flag = false;
		}
	}
	if( table[ nLastIndex ] === 0 && nLastIndex > 0 ) table.length--;
	return flag; // This function should return Boolean( is this odd? )
}


Bigint._strhalf = function( strNumber ) {
	var len = strNumber.length;
	var pos = len % 15 || 15;
	var n = Number( strNumber.substr( 0 , pos ) );

	var flag;
	if( flag = Boolean( n & 1 ) ) n--;
	var strResult = ( n )? String( n / 2 ) : "" ;

	var nSteps = ( len - pos ) / 15;
	for( var i = 0; i < nSteps; i++, pos+=15 ) {
		n = Number( strNumber.substr( pos, 15 ) );
		if( flag ) n += __BIGINT_BIGGER_UNIT;
		if( flag = Boolean( n & 1 ) ) n--;
		strResult += Bigint._padding15( n / 2 );
	}
	return strResult;
}


Bigint._half_typeA = function( strNumber, nAccuracy ) {
	var nZeros = strNumber.length - nAccuracy;
	if( nZeros > 0 ) {
		return Bigint._strhalf( strNumber.substr( 0 , nAccuracy ) ) + Bigint._getStringZeros( nZeros );
	} else {
		return Bigint._strhalf( strNumber );
	}
}

Bigint.div = function( dividend, divisor ) {
	return ( Bigint.qr( dividend, divisor ) )[ 0 ];
}

Bigint.qr = function( dividend, divisor ) {
	var bigDividend = Bigint.serio( dividend );
	var bigDivisor = Bigint.serio( divisor );

	if( bigDivisor.isZero() ) {
		return Bigint._error( "qr" , "divisor is 0" );
	} else if( bigDividend.isZero() ) {
		return [ new Bigint( 0 ), bigDividend ];
	}

	var QR = Bigint._qr( bigDividend, bigDivisor );
	if( bigDividend.isNonNegative ) {
		if( !bigDivisor.isNonNegative ) {
			QR[ 0 ].isNonNegative = false;
			if( !QR[ 1 ].isZero() ) {
				QR[ 0 ]._increment( 1 );
				QR[ 1 ] = Bigint._sub( bigDivisor, QR[ 1 ] );
				QR[ 1 ].isNonNegative = false;
			}
		}
	} else {
		if( bigDivisor.isNonNegative ) {
			QR[ 0 ].isNonNegative = false;
			if( !QR[ 1 ].isZero() ) {
				QR[ 0 ]._increment( 1 );
				QR[ 1 ] = Bigint._sub( bigDivisor, QR[ 1 ] );
			}
		} else { // Negative / Negative
			QR[ 1 ].isNonNegative = QR[ 1 ].isZero();
		}
	}

	return QR;
}

Bigint._qr = function( bigDividend, bigDivisor ) {

	var nDividendLength = bigDividend.getLength();
	var nDivisorLength = bigDivisor.getLength();

	if( nDividendLength < nDivisorLength ) return [ new Bigint( 0 ) , bigDividend ];

	var QR;

	if( __BIGINT_MOZ ) {
		if( nDivisorLength < 14 ) QR = Bigint._qr_typeN( bigDividend.toDecimalString(), bigDivisor.toNumber() );
		else if( nDivisorLength === 14 ) QR = Bigint._qr_typeN_v1( bigDividend.toDecimalString(), bigDivisor.toNumber() );
		else if( nDivisorLength < 295 ) return Bigint._qr_typeF( bigDividend, bigDivisor );
		else if( nDivisorLength < 60000 ) return Bigint._qr_typeS( bigDividend, bigDivisor );
		else return Bigint._qr_typeX( bigDividend, bigDivisor );
	} else {
		if( nDivisorLength < 14 ) {
			QR = Bigint._qr_typeN( bigDividend.toDecimalString(), bigDivisor.toNumber() );
		} else if( nDivisorLength < 295 ) return Bigint._qr_typeF( bigDividend, bigDivisor );
		else if( nDivisorLength < 3500 ) return Bigint._qr_typeS( bigDividend, bigDivisor );
		else return Bigint._qr_typeX( bigDividend, bigDivisor );
	}

	return [ new Bigint(QR[0]), new Bigint(QR[1]) ];

}

Bigint._div = function( bigDividend, bigDivisor ) {
	return Bigint._qr( bigDividend, bigDivisor )[0];
}

Bigint.divmod = Bigint.qr;

Bigint.mod = function( dividend, modulus ) {
	var bigDividend = Bigint.serio( dividend );
	var bigDivisor = Bigint.serio( modulus );

	if( bigDivisor.isZero() || bigDivisor.sign < 0 ) {
		return Bigint._error( "mod" , "modulus is not positive: " + modulus );
	} else if( bigDivisor.isEqualTo( 1 ) ) {
		return new Bigint( 0 );
	} else if( bigDivisor.isEqualTo( 2 ) ) {
		return new Bigint( bigDividend.table[ 0 ] & 1 );
	}

	var bigResult = Bigint._mod( bigDividend, bigDivisor );

	if( !bigDividend.isNonNegative ) bigResult = Bigint._sub( bigDivisor, bigResult );

	return bigResult;
}

Bigint._mod = function( bigDividend, bigDivisor ) {
	var nDividendLength = bigDividend.getLength();
	var nDivisorLength = bigDivisor.getLength();

	if( nDividendLength < nDivisorLength ) return bigDividend.copy();

	if( nDividendLength >= 65536 ) {
		if( nDivisorLength < 65536 ) return Bigint._qr_typeX( bigDividend, bigDivisor )[ 1 ];
		else return Bigint._qr_typeSS( bigDividend, bigDivisor )[ 1 ];
	}

	if( nDivisorLength <= 14 ) {
		var strResult = Bigint._mod_typeN( bigDividend.toDecimalString(), bigDivisor.toNumber() );
		return new Bigint( strResult );
	} else if( nDivisorLength < 295 ) { // 294 + GREEDINESS(14) = 308-digit < 2**1024 = 309 digit
		return Bigint._mod_typeF( bigDividend, bigDivisor );
	} else {
		return Bigint._mod_typeS( bigDividend, nDivisor );
	}
}


///////////////////////////////////////////////////////////////////////////////
//
//	type N
//
///////////////////////////////////////////////////////////////////////////////

Bigint._qr_typeN = function( strDividend, nDivisor ) {
	// nDivisor can be 14-digit or less
	// However, _qr_typeN_v1 is faster for 14-digit
	// This function is fast for small nDivisor.

	var nDividendLength = strDividend.length;
	var strDividendHead = strDividend.substr( 0 , 15 );
	var pos = 15;
	var strQuotient = "";
	var q, r, nDigitsToEat, strRemainderTail;
	while( true ) {
		r = strDividendHead % nDivisor;
		q = ( strDividendHead - r ) / nDivisor;

		strDividendHead = ( r )? String( r ) : "" ;
		if( strQuotient ) strQuotient += Bigint._padding[ strRemainderTail.length ]( q );
		else strQuotient = String( q );
		if( pos >= nDividendLength ) break;
		nDigitsToEat = 15 - strDividendHead.length;
		strRemainderTail = strDividend.substr( pos, nDigitsToEat );
		strDividendHead += strRemainderTail;
		pos += nDigitsToEat;
	}
	if( !strQuotient ) strQuotient = "0";
	if( !strDividendHead ) strDividendHead = "0";
	return [ strQuotient, strDividendHead ];
}

Bigint._mod_typeN = function( strDividend, nDivisor ) {
	// nDivisor should be 14-digit or less
	var nDividendLength = strDividend.length;
	var strDividendHead = strDividend.substr( 0 , 15 );
	var pos = 15;
	while( true ) {
		var r = strDividendHead % nDivisor;
		if( pos >= nDividendLength ) return r;
		strDividendHead = ( r )? String( r ) : "";
		var nDigitsToEat = 15 - strDividendHead.length;
		strDividendHead += strDividend.substr( pos, nDigitsToEat );
		pos += nDigitsToEat;
	}
}

Bigint._isDivisible_typeN = function( strDividend, nDivisor ) {
	var nDividendLength = strDividend.length;
	var strDividendHead = strDividend.substr( 0 , 15 );
	var pos = 15; 
	while( true ) {
		var r = strDividendHead % nDivisor;
		if( pos >= nDividendLength ) return ( ( strDividendHead % nDivisor === 0 ) );
		strDividendHead = ( r )? String( r ) : "" ;
		var nDigitsToEat = 15 - strDividendHead.length;
		strDividendHead += strDividend.substr( pos, nDigitsToEat );
		pos += nDigitsToEat;
	}
}

Bigint._qr_typeN_v1 = function( strDividend, nDivisor ) {
	var nDividendLength = strDividend.length;
	var nDivisorLength = String( nDivisor ).length;
	var strDividendHead = strDividend.substr( 0 , nDivisorLength );
	var strQuotient = "";
	for( var pos = nDivisorLength - 1; pos < nDividendLength; pos++ ) {
		if( Number( strDividendHead ) < nDivisor ) {
			if( strQuotient ) strQuotient += "0";
			if( pos === nDividendLength - 1 ) break;
			strDividendHead += strDividend.charAt( pos + 1 );
			continue;
		}
		var r = strDividendHead % nDivisor;
		var strRemainderHead = ( r )? String( r ) : "" ;
		var q = ( strDividendHead - r ) / nDivisor;
		strQuotient += String( q );
		strDividendHead = strRemainderHead + strDividend.charAt( pos + 1 );
	}
	if( !strQuotient ) strQuotient = "0";
	if( !strDividendHead ) strDividendHead = "0";
	return [ strQuotient, strDividendHead ];
}

///////////////////////////////////////////////////////////////////////////////
//
//	type F
//
///////////////////////////////////////////////////////////////////////////////


Bigint._qr_typeF = function( bigDividend, bigDivisor ) { // v3

	var strDividend = bigDividend.toDecimalString();
	var strDivisor = bigDivisor.toDecimalString();
	var nDivisorLength = strDivisor.length;
	var nDividendLength = strDividend.length;


	if( nDividendLength <= nDivisorLength && Bigint._strcmp( strDividend, strDivisor ) < 0 ) {
		return [ new Bigint( 0 ), bigDividend ];
	}

	var GREEDINESS = 14;
	var strDividendHead = strDividend.substr( 0 , strDivisor.length + GREEDINESS );
	var pos = nDivisorLength + GREEDINESS;
	var strResultQ = "";
	var fQuotient, bigQuotient, bigProduct, strProduct, strRemainderHead;
	var correction, nDigitsToEat, tmppos;

	for( var dummy=0; dummy < 5000; dummy++ ) {

		fQuotient = strDividendHead / strDivisor;

		bigQuotient = new Bigint( Math.floor( fQuotient ) );

		bigProduct = Bigint._mul( bigDivisor, bigQuotient );
		strProduct = bigProduct.toDecimalString();


		strRemainderHead = Bigint._strsub( strDividendHead , strProduct );

		correction = 0;

		while( strRemainderHead === undefined ) {

			correction++;
			bigQuotient._decrement( 1 );
			strProduct = Bigint._strsub( strProduct, strDivisor );
			strRemainderHead = Bigint._strsub( strDividendHead , strProduct );
			if( correction > 10 ) {
				return Bigint._error( "_qr_typeF", "negative auto-correction failed: " + bigDividend + " / " + bigDivisor );
			}
		}

		if( !correction ) {
			while( Bigint._strcmp( strRemainderHead, strDivisor ) >= 0 ) {
				correction++;
				bigQuotient._increment( 1 );
				strRemainderHead = Bigint._strsub( strRemainderHead , strDivisor );
				if( correction > 10 ) {
					return Bigint._error( "_qr_typeF", "failed to auto-correct: " + bigDividend + " / " );
				}
			}
		}

		strResultQ += bigQuotient.toDecimalString();


//-------------- NEW START --- (beta 21)
		var nLeftoverDigits = strRemainderHead.length;

		if( !nLeftoverDigits ) {
			while( "0" === strDividend.charAt( pos ) ) {
				strResultQ += "0";
				pos++;
			}
		}
		nDigitsToEat = ( nDivisorLength + GREEDINESS ) - nLeftoverDigits;
		strDividendHead = strRemainderHead + strDividend.substr( pos , nDigitsToEat );
		pos += nDigitsToEat;

		var max = strDividendHead.length;
		for( var tmppos = nLeftoverDigits + 1; tmppos <= max; tmppos++ ) {
			if( Bigint._strcmp( strDividendHead.substring( 0, tmppos ), strDivisor ) >= 0 ) {
				break;
			} else {
				strResultQ += "0";
			}
		}
//-------------- NEW END ---

		if( pos >= nDividendLength && Bigint._strcmp( strDividendHead, strDivisor ) < 0 ) {
			 return [ new Bigint( strResultQ ), new Bigint( strDividendHead ) ];
		}
	}

	return Bigint._error("_qr_typeF", "too many loops for " + bigDividend + " / " + bigDivisor );
}


Bigint._mod_typeF = function( bigDividend, bigDivisor ) { // v3
	var strDividend = bigDividend.toDecimalString();
	var strDivisor = bigDivisor.toDecimalString();
	var nDividendLength = strDividend.length;
	var nDivisorLength = strDivisor.length;

//{BUG}------------ beta27 deleted
//	if( pos >= nDividendLength && Bigint._strcmp( strDividendHead , strDivisor ) < 0 )
//--------------------------------
	if( nDividendLength < nDivisorLength || nDividendLength === nDivisorLength && Bigint._strcmp( strDividend, strDivisor ) < 0 ) {
//Hotfix 2 DEL:	return bigDivisor;
//Hotfix 2 INS:
		return new Bigint( strDividend );
	}


	var GREEDINESS = 14; // 15 might be slightly better for middle-sized bigDivisor ( like 20 ~ 40 digit )

	var strDividendHead = strDividend.substr( 0 , strDivisor.length + GREEDINESS );
	var pos = nDivisorLength + GREEDINESS;
	var fQuotient, bigQuotient, bigProduct, strProduct, strRemainderHead;
	var q, fractional, correction, nDigitsToEat;

	for( var dummy=0; dummy < 5000; dummy++ ) {
		// Because of floating-point arithmetic, q may be inaccurate (sometimes error=1, rarely error=2)
		// If q is too small, strRemainderHead will be > strDivisor, but no problem in going on.
		// If q is too big, we should correct q later.
		fQuotient = strDividendHead / strDivisor;
		q = Math.floor( fQuotient );

		 //	This trick does work for Miller-Rabin test
		 //	 where you'd get bigDivident congruent to -1 modulo bigDivisor.
		 //	Without this, you are likely to get 'correction=1' (see below) many times, which will affect the speed.
		 // This trick does not affect the spped because usually it is unlikely to have fractional < 0.00001
 		fractional = fQuotient - q;
		if( fractional < 0.00001 && q > 1 ) { q--; }

		bigQuotient = new Bigint( q );
		bigProduct = Bigint._mul( bigDivisor, bigQuotient );
		strProduct = bigProduct.toDecimalString();
		strRemainderHead = Bigint._strsub( strDividendHead , strProduct );

		// If strProduct is bigger and cannot be subtracted from strDividendHead,
		// _strsub returns undefined (This doesn't happen too often)
		correction = 0;
		while( strRemainderHead === undefined ) {
			correction++;

			// Let q be q-1, then strProduct will be this:
			strProduct = Bigint._strsub( strProduct, strDivisor );

			// Probably now we can subtract strProduct from strDividendHead
			strRemainderHead = Bigint._strsub( strDividendHead , strProduct );

			if( correction > 10 ) return Bigint._error( "_mod_typeF", "failed to auto-correct: " + bigDividend + " / " + bigDivisor );
		}

		nDigitsToEat = strDivisor.length + GREEDINESS - strRemainderHead.length;
		strDividendHead = strRemainderHead + strDividend.substr( pos , nDigitsToEat );
		pos += nDigitsToEat;

		if( pos >= nDividendLength && Bigint._strcmp( strDividendHead, strDivisor ) < 0 ) {
			return new Bigint( strDividendHead );
		}
	}

	return Bigint._error("_mod_typeF", "too many loops for " + bigDividend + " / " + bigDivisor );
}

Bigint._div_typeF = function( bigA, bigB ) {
	return Bigint._qr_typeF( bigA, bigB )[0];
}

///////////////////////////////////////////////////////////////////////////////
//
//	type S ( Divisor : 295-digit or more, 65535-digit or less )
//
///////////////////////////////////////////////////////////////////////////////

Bigint._qr_typeS = function( bigDividend, bigDivisor ) {
	var strDividend = bigDividend.toDecimalString();
	var strDivisor = bigDivisor.toDecimalString();
	var nDividendLength = strDividend.length;
	var nDivisorLength = strDivisor.length;

	if( nDividendLength < nDivisorLength || nDividendLength === nDivisorLength && Bigint._strcmp( strDividend, strDivisor ) < 0 ) {
		return [ new Bigint( 0 ), bigDividend ];
	}

	var DIVISOR_HEAD_LENGTH = 16;
	var strDivisorHead = strDivisor.substr( 0 , DIVISOR_HEAD_LENGTH );

	var GREEDINESS = 14;
		// Can't be 15. Unlike _mod_typeF, fQuotient is triple-lossy (denominator, numerator & float ).
		// High GREEDINESS would produce such a big error that couldn't be auto-corrected.
	var DIMINISHED_DIVIDEND_HEAD_LENGTH = DIVISOR_HEAD_LENGTH + GREEDINESS;

	var strDividendHead = strDividend.substr( 0 , strDivisor.length + GREEDINESS );
	var pos = nDivisorLength + GREEDINESS;
	var cuteness = pos - strDividendHead.length; // always 0 unless strDividend is so short that pos > its length
	var strDividendHeadDiminished;
	var strResultQ = "";
	var fQuotient, bigQuotient, bigProduct, strProduct, strRemainderHead;
	var correction, nDigitsToEat, tmppos;

	for( var dummy=0; dummy < 5000; dummy++ ) {
		strDividendHeadDiminished = strDividendHead.substr( 0 , DIMINISHED_DIVIDEND_HEAD_LENGTH - cuteness );

		fQuotient = strDividendHeadDiminished / strDivisorHead;
		bigQuotient = new Bigint( Math.floor( fQuotient ) );
		bigProduct = Bigint._mul( bigDivisor, bigQuotient );
		strProduct = bigProduct.toDecimalString();
		strRemainderHead = Bigint._strsub( strDividendHead , strProduct );

		correction = 0;
		while( strRemainderHead === undefined ) {
			correction++;
			bigQuotient._decrement( 1 );
			strProduct = Bigint._strsub( strProduct, strDivisor );
			strRemainderHead = Bigint._strsub( strDividendHead , strProduct );
			if( correction > 10 ) return Bigint._error( "_qr_typeS", "negative auto-correction failed: " + bigDividend + " / " + bigDivisor );
		}

		if( !correction ) {
			while( Bigint._strcmp( strRemainderHead, strDivisor ) >= 0 ) {
				correction++;
				bigQuotient._increment( 1 );
				strRemainderHead = Bigint._strsub( strRemainderHead , strDivisor );
				if( correction > 10 ) return Bigint._error( "_qr_typeS", "positive auto-correction failed: " + bigDividend + " / " + bigDivisor );
			}
		}
		strResultQ += bigQuotient.toDecimalString();

//-------------- NEW START --- (beta 21)
		var nLeftoverDigits = strRemainderHead.length;

		if( !nLeftoverDigits ) {
			while( "0" === strDividend.charAt( pos ) ) {
				strResultQ += "0";
				pos++;
			}
		}
		nDigitsToEat = ( nDivisorLength + GREEDINESS ) - nLeftoverDigits;
		strDividendHead = strRemainderHead + strDividend.substr( pos , nDigitsToEat );
		pos += nDigitsToEat;

		var max = strDividendHead.length;
		for( var tmppos = nLeftoverDigits + 1; tmppos <= max; tmppos++ ) {
			if( Bigint._strcmp( strDividendHead.substring( 0, tmppos ), strDivisor ) >= 0 ) {
				break;
			} else {
				strResultQ += "0";
			}
		}
//-------------- NEW END ---



		if( pos >= nDividendLength && Bigint._strcmp( strDividendHead, strDivisor ) < 0 ) {
			return [ new Bigint( strResultQ ) , new Bigint( strDividendHead ) ];
		}

		cuteness = nDivisorLength + GREEDINESS - strDividendHead.length;
	}

	return Bigint._error( "_qr_typeS", "too many loops for " + bigDividend + " / " + bigDivisor );
}


Bigint._mod_typeS = function( bigDividend, bigDivisor ) {
	var strDividend = bigDividend.toDecimalString();
	var strDivisor = bigDivisor.toDecimalString();
	var nDividendLength = strDividend.length;
	var nDivisorLength = strDivisor.length;
	if( nDividendLength < nDivisorLength || nDividendLength === nDivisorLength && Bigint._strcmp( strDividend, strDivisor ) < 0 ) {
		return bigDivisor;
	}
	var DIVISOR_HEAD_LENGTH = 16;
	var strDivisorHead = strDivisor.substr( 0 , DIVISOR_HEAD_LENGTH );
	var GREEDINESS = 14;
	var DIMINISHED_DIVIDEND_HEAD_LENGTH = DIVISOR_HEAD_LENGTH + GREEDINESS;
	var strDividendHead = strDividend.substr( 0 ,  nDivisorLength + GREEDINESS );
	var pos = nDivisorLength + GREEDINESS;
	var cuteness = pos - strDividendHead.length;
	var strDividendHeadDiminished;
	var fQuotient, bigQuotient, bigProduct, strProduct, strRemainderHead;
	var correction, nDigitsToEat;

	for( var dummy=0; dummy < 5000; dummy++ ) {
		strDividendHeadDiminished = strDividendHead.substr( 0 , DIMINISHED_DIVIDEND_HEAD_LENGTH - cuteness );
		fQuotient = strDividendHeadDiminished / strDivisorHead;
		bigQuotient = new Bigint( Math.floor( fQuotient ) );
		bigProduct = Bigint._mul( bigDivisor, bigQuotient );
		strProduct = bigProduct.toDecimalString();
		strRemainderHead = Bigint._strsub( strDividendHead , strProduct );
		correction = 0;
		while( strRemainderHead === undefined ) {
			correction++;
			strProduct = Bigint._strsub( strProduct, strDivisor );
			strRemainderHead = Bigint._strsub( strDividendHead , strProduct );
			if( correction > 10 ) return Bigint._error("_mod_typeS", "failed to auto-correct: " + bigDividend + " / " + bigDivisor);
		}
		nDigitsToEat = nDivisorLength + GREEDINESS - strRemainderHead.length;
		strDividendHead = strRemainderHead + strDividend.substr( pos , nDigitsToEat );
		pos += nDigitsToEat;

		if( pos >= nDividendLength && Bigint._strcmp( strDividendHead, strDivisor ) < 0 ) {
			return new Bigint( strDividendHead );
		}
		cuteness = nDivisorLength + GREEDINESS - strDividendHead.length;
	}
	return Bigint._error( "_mod_typeS" , "too many loops for " + bigDividend + " / " + bigDivisor );
}

Bigint._div_typeS = function( bigA, bigB ) {
	return Bigint._qr_typeS( bigA, bigB )[ 0 ];
}


///////////////////////////////////////////////////////////////////////////////
//
//	misc
//
///////////////////////////////////////////////////////////////////////////////

Bigint._div_typeA = function( strDividend, strDivisor, nAccuracy ) {
	var strA = strDividend.substr( 0 , nAccuracy * 2 );
	var strB = strDivisor.substr( 0 , nAccuracy );
	var strQ = ( nAccuracy < 295 )? Bigint._strdiv_typeF( strA , strB ) : Bigint._strdiv_typeS( strA , strB ) ;
	var nDigits = strDividend.length - strDivisor.length;
	if( Bigint._strcmp2( strA , strB ) >= 0 ) nDigits++;
	var nZeros = nDigits - strQ.length;
	if( nZeros > 0 ) return strQ + Bigint._getStringZeros( nZeros );
	else return strQ;
}



Bigint._strdiv_typeS = function( strDividend, strDivisor, bListContext ) {
	var nDividendLength = strDividend.length;
	var nDivisorLength = strDivisor.length;
	var bigDivisor = new Bigint( strDivisor );
	var DIVISOR_HEAD_LENGTH = 16;
	var strDivisorHead = strDivisor.substr( 0 , DIVISOR_HEAD_LENGTH );
	var GREEDINESS = 14;
	var DIMINISHED_DIVIDEND_HEAD_LENGTH = DIVISOR_HEAD_LENGTH + GREEDINESS;
	var strDividendHead = strDividend.substr( 0 , nDivisorLength + GREEDINESS );
	var pos = nDivisorLength + GREEDINESS;
	var cuteness = pos - strDividendHead.length; // always 0 unless strDividend is so short that pos > its length

	var strResultQ = "";

	var strDividendHeadDiminished;
	var fQuotient, bigQuotient, bigProduct, strProduct, strRemainderHead;
	var correction, nDigitsToEat, tmppos;

	for( var dummy=0; dummy < 5000; dummy++ ) {
		strDividendHeadDiminished = strDividendHead.substr( 0 , DIMINISHED_DIVIDEND_HEAD_LENGTH - cuteness );
		fQuotient = strDividendHeadDiminished / strDivisorHead;
		bigQuotient = new Bigint( Math.floor( fQuotient ) );
		strProduct = Bigint._mul( bigDivisor, bigQuotient ).toDecimalString();
		strRemainderHead = Bigint._strsub( strDividendHead , strProduct );
		correction = 0;
		while( strRemainderHead === undefined ) {
			correction++;
			bigQuotient._decrement( 1 );
			strProduct = Bigint._strsub( strProduct, strDivisor );
			strRemainderHead = Bigint._strsub( strDividendHead , strProduct );
			if( correction > 10 ) {
				return Bigint._error( "_strdiv_typeS", "negative auto-correction failed" );
			}
		}
		if( !correction ) {
			while( Bigint._strcmp( strRemainderHead, strDivisor ) >= 0 ) {
				correction++;
				bigQuotient._increment( 1 );
				strRemainderHead = Bigint._strsub( strRemainderHead , strDivisor );
				if( correction > 10 ) {
					return Bigint._error( "_strdiv_typeS", "positive auto-correction failed" );
				}
			}
		}
		strResultQ += bigQuotient.toDecimalString();

//-------------- NEW START --- (beta 21)
		var nLeftoverDigits = strRemainderHead.length;

		if( !nLeftoverDigits ) {
			while( "0" === strDividend.charAt( pos ) ) {
				strResultQ += "0";
				pos++;
			}
		}
		nDigitsToEat = ( nDivisorLength + GREEDINESS ) - nLeftoverDigits;
		strDividendHead = strRemainderHead + strDividend.substr( pos , nDigitsToEat );
		pos += nDigitsToEat;

		var max = strDividendHead.length;
		for( var tmppos = nLeftoverDigits + 1; tmppos <= max; tmppos++ ) {
			if( Bigint._strcmp( strDividendHead.substring( 0, tmppos ), strDivisor ) >= 0 ) {
				break;
			} else {
				strResultQ += "0";
			}
		}
//-------------- NEW END ---



		if( pos >= nDividendLength && Bigint._strcmp( strDividendHead, strDivisor ) < 0 ) {
			 if( bListContext ) return [ strResultQ , strDividendHead ];
			 else return strResultQ;
		}
		cuteness = nDivisorLength + GREEDINESS - strDividendHead.length;
	}
	return Bigint._error("_strdiv" , "too many loops" );
}


Bigint._strdiv_typeF = function( strDividend, strDivisor, bListContext ) {
	var nDividendLength = strDividend.length;
	var nDivisorLength = strDivisor.length;
	var bigDivisor = new Bigint( strDivisor );
	var GREEDINESS = 14;
	var strDividendHead = strDividend.substr( 0 , nDivisorLength + GREEDINESS );
	var pos = nDivisorLength + GREEDINESS;
	var strResultQ = "";

	var fQuotient, bigQuotient, bigProduct, strProduct, strRemainderHead;
	var correction, nDigitsToEat, tmppos;

	for( var dummy=0; dummy < 5000; dummy++ ) {
		fQuotient = strDividendHead / strDivisor;
		bigQuotient = new Bigint( Math.floor( fQuotient ) );
		bigProduct = Bigint._mul( bigDivisor, bigQuotient )
		strProduct = bigProduct.toDecimalString();
		strRemainderHead = Bigint._strsub( strDividendHead , strProduct );
		correction = 0;
		while( strRemainderHead === undefined ) {
			correction++;
			bigQuotient._decrement( 1 );
			strProduct = Bigint._strsub( strProduct, strDivisor );
			strRemainderHead = Bigint._strsub( strDividendHead , strProduct );
			if( correction > 10 ) {
				return Bigint._error( "_strdiv_typeF", "negative auto-correction failed" );
			}
		}
		if( !correction ) {
			while( Bigint._strcmp( strRemainderHead, strDivisor ) >= 0 ) {
				correction++;
				bigQuotient._increment( 1 );
				strRemainderHead = Bigint._strsub( strRemainderHead , strDivisor );
				if( correction > 10 ) {
					return Bigint._error( "_strdiv_typeF", "positive auto-correction failed" );
				}
			}
		}
		strResultQ += bigQuotient.toDecimalString();

//-------------- NEW START --- (beta 21)
		var nLeftoverDigits = strRemainderHead.length;

		if( !nLeftoverDigits ) {
			while( "0" === strDividend.charAt( pos ) ) {
				strResultQ += "0";
				pos++;
			}
		}
		nDigitsToEat = ( nDivisorLength + GREEDINESS ) - nLeftoverDigits;
		strDividendHead = strRemainderHead + strDividend.substr( pos , nDigitsToEat );
		pos += nDigitsToEat;

		var max = strDividendHead.length;
		for( var tmppos = nLeftoverDigits + 1; tmppos <= max; tmppos++ ) {
			if( Bigint._strcmp( strDividendHead.substring( 0, tmppos ), strDivisor ) >= 0 ) {
				break;
			} else {
				strResultQ += "0";
			}
		}
//-------------- NEW END ---


		if( pos >= nDividendLength && Bigint._strcmp( strDividendHead , strDivisor ) < 0 ) {
			if( bListContext ) return [ strResultQ , strDividendHead ];
			else return strResultQ;
		}
	}
	return Bigint._error("_strdiv_typeF", "too many loops" );
}


Bigint._qr_typeX = function( bigDividend, bigDivisor ) {

	var originalTable = bigDividend.table;
	var META_GREEDINESS = 120;
	var nWorkSize = Math.ceil( originalTable.length / META_GREEDINESS );
	var pQuotientParts = new Array( nWorkSize ); // Resulted Quotient Strings, like 60000 digits for each

	var nDividendLength = bigDividend.getLength(); // Can be >= 65536
	var strDivisor = bigDivisor.toDecimalString();
	var nDivisorLength = strDivisor.length; // must be < 65536

	var FN_STRDIV = ( nDivisorLength < 295 )? Bigint._strdiv_typeF : Bigint._strdiv_typeS ;
	var nLastIndex = originalTable.length - 1;

	var tmppos, tmpstr;

	var nQuotientLength = 0;

	var strDividendHead = "";

	for( var i = 0; i < nWorkSize; i++ ) {
if(i%4===0) {
	_progress_ = Math.floor( i / nWorkSize * 100 ) + "% ";
	if(__BIGINT_MOZ) document.__BIGINT_HTML_FORM.output.value += _progress_ ;
	else defaultStatus += _progress_ ;
}
		var nStartIndex = nLastIndex - i * META_GREEDINESS;
		var nEndIndex = nStartIndex - META_GREEDINESS + 1;
		if( nEndIndex < 0 ) nEndIndex = 0;
		var nWorkLength = nStartIndex - nEndIndex + 1;

		if( strDividendHead ) {
			strDividendHead += __BIGINT_FN_PADDING( originalTable[ nStartIndex ] );
		} else {
			strDividendHead = String( originalTable[ nStartIndex ] );
		}

		for( var j = 1; j < nWorkLength; j++ ) {
			strDividendHead += __BIGINT_FN_PADDING( originalTable[ nStartIndex - j ] );
		}

		if( i >= 1 ) {
			var endpos = strDividendHead.length;
			for( ; tmppos < endpos; tmppos++ ) {
				tmpstr += strDividendHead.charAt( tmppos );
				if( Bigint._strcmp( tmpstr, strDivisor ) >= 0 ) break;
				pQuotientParts[ i - 1 ] += "0";
				nQuotientLength++;
			}
		}
		tmpstr = null;

		var tmpQR = FN_STRDIV( strDividendHead, strDivisor, true );

		pQuotientParts[ i ] = tmpQR[ 0 ];
		strDividendHead = ( tmpQR[ 1 ] === "0" )? "" : tmpQR[ 1 ] ;

		tmppos = strDividendHead.length;
		tmpstr = strDividendHead;

		tmpQR.length = 0;
		tmpQR = null;
		nQuotientLength += pQuotientParts[ i ].length;

	}
	strDivisor = null;

	var nTableSizeQ = Math.ceil( nQuotientLength / 7 );


	var tableQ = new Array( nTableSizeQ );
	var nLastItemLength = nQuotientLength % 7;
	if( nLastItemLength === 0 ) nLastItemLength = 7;
	var idx = nTableSizeQ - 1; // last index


	tableQ[ idx-- ] = Number( pQuotientParts[ 0 ].substr( 0 , nLastItemLength ) );
	var pos = nLastItemLength; // 1 2 3 4 5 6 or 7
	for( var i = 0; i < nWorkSize; i++ ) {
		var len = pQuotientParts[ i ].length - pos;
		var nSteps = Math.floor( len / 7 );
		var r = len - nSteps * 7;
		for( var j = 0; j < nSteps; j++ ) {
			tableQ[ idx-- ] = Number( pQuotientParts[ i ].substr( pos, 7 ) );
			pos += 7;
		}
		if( r === 0 ) { // Especially this should be true for the last i
			pos = 0;
		} else {
			var tmpstr = pQuotientParts[ i ].substr( pos, r );
			pos = 7 - r; // 1 2 3 4 5 or 6
			tableQ[ idx-- ] = Number( tmpstr + pQuotientParts[ i + 1 ].substr( 0 , pos ) );
		}

		pQuotientParts[ i ] = null;
	}

	pQuotientParts.length = 0;
	pQuotientParts = null;
	var bigR = new Bigint( strDividendHead );
	strDividendHead = null;
	var bigQ = new Bigint();
	bigQ.table = tableQ;

if(__BIGINT_MOZ) document.__BIGINT_HTML_FORM.output.value += "100% :D" ;
else defaultStatus += "100% :D" ;

	return [ bigQ , bigR ];
}