// log.js

Bigint.prototype.log = function( input ) {
	var base = Number( input );
	if( isNaN( base ) || base <= 0 ) return Bigint._error( "log" , "Bad input: " + input );
	var oFloatEx = this.toFloatEx();
	var ln = oFloatEx.power * Math.LN10 + Math.log( oFloatEx.number );
	if( base === undefined ) return ln;
	else return ln / Math.log( base );
}

Bigint.prototype.ln = function() {
	var oFloatEx = this.toFloatEx();
	return oFloatEx.power * Math.LN10 + Math.log( oFloatEx.number );
}

Bigint.prototype.getBitWidthS = function( input ) {
	if( !this.isNonNegative ) return Bigint._error( "getBitWidthS", "This is negative: " + this );
	if( this.table.length <= __BIGINT_JS_COMPATIBLE_MAX_CELLS ) {
		var strLog2 = String( Math.log( this.toDecimalString() ) / Math.LN2 );
	} else {
		var oFloatEx = this.toFloatEx();
		var ln = oFloatEx.power * Math.LN10 + Math.log( oFloatEx.number );

		strLog2 = String( ln / Math.log( 2 ) );
	}

	if( strLog2.indexOf(".") < 0 ) strLog2 += ".";

	strLog2 += Bigint._getStringZeros( 16 );

	var pos = strLog2.indexOf(".") + 1;

	var d = Number( input );

	if( !isNaN( d ) && d >= 1 ) {
		if( d > 16 ) d = 16;
		else d = Math.floor( d );
		return strLog2.substring( 0, pos + d );
	} else {
		return strLog2.substring( 0, pos - 1 );
	}
}

Bigint.prototype.getBitWidth = function( d ) {
	return Number( this.getBitWidthS( d ) );
}

Bigint.log2 = function( n ) {
	var bigNumber = Bigint.serio( n );
	return bigNumber.getBitWidth();
}

Bigint._log2 = function( bigNumber ) {
	return bigNumber.getBitWidth();
}


Bigint.prototype.toFloatEx = function() {
	if( this.table.length < 4 ) {
		var strFloatNumber = "0." + this.toDecimalString();
	} else {
		var table = this.table;
		var lastIndex = table.length - 1;
		var strFloatNumber = "0." + table[lastIndex]
			+ __BIGINT_FN_PADDING( table[lastIndex-1] )
		 	+ __BIGINT_FN_PADDING( table[lastIndex-2] )
			+ __BIGINT_FN_PADDING( table[lastIndex-3] );
	}

	var oResult = new Object();

	oResult.isNonNegative = this.isNonNegative;
	oResult.number = strFloatNumber * 10;
	oResult.power = this.getLength() - 1;
	return oResult;
}
