// Original Code in C [ mt19937int.c ] under the Artistic license, dated 2001-04-02, by Matsumoto, etc.
// http://www.math.keio.ac.jp/~matumoto/mt19937.tar
// Translated into JavaScript, keeping the same logic, on 2003-12-25, by www.faireal.net, copyleft.
// Check http://www.faireal.net/demo/ for the newest version.

MersenneTwister = function( nSeed ) {
	if( nSeed === undefined ) nSeed = 4357;
	else nSeed >>>= 0;

	this.pInteger = new Array( 624 );

    for ( var i = 0; i < 624; i++ ) {
	   	this.pInteger[i] = ( nSeed & 0xffff0000 ) >>> 0;
       	nSeed = ( 69069 * nSeed + 1 ) >>> 0;
       	this.pInteger[i] = ( this.pInteger[i] | ( nSeed & 0xffff0000 ) >>> 16 ) >>> 0;
       	nSeed = ( 69069 * nSeed + 1 ) >>> 0;
   	}
	this._initialize();
}

MersenneTwister.prototype.getRandomUINT32 = function() {
    if ( this.index > 623 ) this._initialize();
//_debug("getRandomUINT32: this.index=" + this.index);
    var y = this.pInteger[ this.index++ ];

    y ^= y >>> 11, y ^= (( y << 7 ) & 0x9d2c5680) >>> 0,
    y ^= (( y << 15 ) & 0xefc60000) >>> 0, y ^= y >>> 18;
    return y >>> 0; 
}
MersenneTwister.prototype.toString = MersenneTwister.prototype.getRandomUINT32;

MersenneTwister.prototype._initialize = function() {
	var i;
    for( i = 0; i < 227; i++ ) this.pInteger[i] = this._jumble( i, i+1, i+397 );
    for( i = 227; i < 623; i++ ) this.pInteger[i] = this._jumble( i, i+1, i-227 );
    this.pInteger[623] = this._jumble( 623, 0, 396 );
    this.index = 0;
}

MersenneTwister.prototype._jumble = function( nIndex, nIndex2, nIndex3 ) {
	var tmp = ( this.pInteger[ nIndex ] & 0x80000000 ) | ( this.pInteger[ nIndex2 ] & 0x7fffffff );
	var tmp2 = ( tmp & 0x1 )? 0x9908b0df : 0 ;
	return ( this.pInteger[ nIndex3 ] ^ ( tmp >>> 1 ) ^ tmp2 ) >>> 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// These are not in the original

MersenneTwister.prototype.getRandomUINT32Fluctuated = function( nMoreEntropy ) {
	if( nMoreEntropy ) {
		if( nMoreEntropy === 0x10000 ) {
			alert("Randomizer: Just click OK.");
		} else {
			var nDummy = Math.floor( nMoreEntropy );
			nDummy %= 10;
			if( nDummy < 2 ) nDummy = 2;
			var nTmpRand = Math.floor( Math.random() * 200 );
			var strDummy = Bigint.pow( nDummy , nTmpRand ).toString(2);
			if( strDummy.indexOf("`") > -1 ) Bigint._debug( strDummy ); // never true
			_debug( "Entropy", strDummy.length );
		}
	}

	var d = new Date;
	var strTime = "0"+ d.getTime()%100;
	var n = Number( strTime.substr( strTime.length - 2, 1 ) );
	this.index += n;
	return this.getRandomUINT32();
}

MersenneTwister.prototype.getRandomBigintOdd = function( nDigits ) {
	if( typeof nDigits !== "number" || nDigits < 1 || nDigits > 10000 ) {
		return Bigint._error( "MersenneTwister.prototype.getBigintOdd" , "Input is not number, too small, or too big: " + nDigits );
	}

	var strResult = "";
	for( var i=0; i<nDigits; i++ ) {
		var d = new Date;
		var strTime = "0"+ d.getTime()%100;
		var n = Number( strTime.substr( strTime.length - 2 , 1 ) );
		var m = strTime.substr( strTime.length - 2 ) % 6 + 2; // 2 3 4 5 6 7

		this.index += n;
		var strRand = "0000000" + this.getRandomUINT32();
		var x = strRand.substr( strRand.length - m , 1 );

		if( i === 0 && x === "0" ) {
			i = -1;
			continue;
		} else if( i === nDigits - 1 && !( x & 1 ) ) { // x = 0 2 4 6 8
			x = String( Number( x ) + 1 );
		}
		strResult += x;
	}
	return new Bigint(strResult);
}

// The first nHead digits will be kept. The rest will be randomized.
// TYPICAL INPUT: Bigint with a lot of trailing zeros, like 60 2770759 3737327 0000000 (23-digit)
// TYPICAL OUTPUT: The trailing-zero part randomized, like: 60 2770759 3737508 9540617 (23-digit)
MersenneTwister.prototype._randomizeTail = function( bigInput, nHead ) {
	if( nHead === undefined ) nHead = 4;
	var strInput = bigInput.toDecimalString();
	if( strInput.length <= nHead ) return bigInput.copy();
	var strHead = strInput.substr( 0 , nHead );
	var nDigitsNeeded = strInput.length - nHead;
	var bigTail = this.getRandomBigintOdd( nDigitsNeeded );
	var strOutput = strHead + bigTail.toDecimalString();
	return new Bigint( strOutput );
}